embed

package
v1.1.1-0...-18dfa52 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Jun 10, 2025 License: Apache-2.0 Imports: 4 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func MirrorResource

func MirrorResource(resPath, dstPath string) error

MirrorResource copies an embedded resource to the given destination path, retaining (mirroring) its full path.

Types

type Resource

type Resource struct {
	Name    string
	Path    string
	Content []byte
}

func GetPreset

func GetPreset(name string) (*Resource, error)

GetPreset returns a preset with a given name. If preset is not found, an error is returned.

func GetResource

func GetResource(filePath string) (*Resource, error)

GetResource returns an embedded resource on a given path. If the resource is not found, an error is thrown.

func GetTemplate

func GetTemplate(tplPath string) (*Resource, error)

GetResource returns a resource on a given path. If resource is not found, an error is thrown

func Presets

func Presets() ([]Resource, error)

Presets returns a list of all available presets.

Jump to