exec

package
v1.1.1-0...-18dfa52 Latest Latest
Warning

This package is not in the latest version of its module.

Go to latest
Published: Jun 10, 2025 License: Apache-2.0 Imports: 9 Imported by: 0

Documentation

Index

Constants

This section is empty.

Variables

This section is empty.

Functions

func NewLocalClient

func NewLocalClient() localClient

NewLocalClient initializes a client for running local commands.

func NewSSHClient

func NewSSHClient(user string, host string) remoteClient

NewSSHClient initializes a new remote SSH client.

func Run

func Run(command string, args ...string) error

Run is a shorthand for running the given command locally.

func RunCtx

func RunCtx(ctx context.Context, command string, args ...string) error

RunCtx is a shorthand for running the given command locally.

Types

type Client

type Client interface {
	Run(command string, args ...string) error
	RunCtx(ctx context.Context, command string, args ...string) error
	Close() error
}

Jump to