Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
This section is empty.
Functions ¶
func MirrorResource ¶
MirrorResource copies an embedded resource to the given destination path, retaining (mirroring) its full path.
Types ¶
type Resource ¶
func GetPreset ¶
GetPreset returns a preset with a given name. If preset is not found, an error is returned.
func GetResource ¶
GetResource returns an embedded resource on a given path. If the resource is not found, an error is thrown.
func GetTemplate ¶
GetResource returns a resource on a given path. If resource is not found, an error is thrown
Click to show internal directories.
Click to hide internal directories.